
'''
This file is automatically generated; Do not edit it. :)
'''
VERSION_INFO = {
    'final': True,
    'version': '1.29.0rc2',
    'branch_nick': 'master',
    'revision_id': '26259038b77f79912edc48252037fb3675f34c96',
    'revno': 11845
}
